/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package SDR;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import static java.lang.Math.log;
import java.util.ArrayList;


public class Bscore {

    ReadAlignment alignment;
    String fileDichletMixture;
    ArrayList<ArrayList<Double>> results;
    int cluster;
    double scoreresiduconserver = 0;

    public Bscore(ReadAlignment alignment, String fileDichletMixture, int cluster) {
        this.alignment = alignment;
        this.fileDichletMixture = fileDichletMixture;
        results = new ArrayList();
        this.cluster = cluster;

    }
   
    
    /**
     * calculate the BILD score of each column in the alignment using columnscore()
     *
     * @return BILD score of each column in the alignment
     */
    ArrayList<Double> scoreColumns() {
        ArrayList<Double> columnscores = new ArrayList();
        double score;
        int sizesequence = (((alignment.sequenceArray).get(1)).Seq).size();
        for (int c = 0; c < sizesequence; c++) {
            score = columnscore(c, cluster);
            //System.out.println("CBS"+" "+cluster+" "+c+" "+score+" ");
            columnscores.add(score);
        }
        return columnscores;
    }

    /**
     * calculate the column score for 1 cluster
     *
     * @param number
     * @return column score 
     */
    

    
        double columnscore(int number, int cluster) {
        DirichletMixture dirichletMixture = new DirichletMixture(fileDichletMixture);
 

        // initialise the score(to 1 because we multiply)
        double score = 1;
        int n = 0;
        int ng = 0;
        
        //multiply by the probability of each residue
        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {
                String c = sequence.Seq.get(number);
                double a = dirichletMixture.Prob((String) c, sequence.Weight, true);
                score *= a;
                if ((".".equals(c)) || ("-".equals(c))) ng++;
                n++;
            }
        }
        double nscore = normalisescore(score, n-ng);
        if(n-ng<3) nscore = 0.0;

        return nscore;
    }
    
    
    /**
     * Normalise le score 
     * @param score
     * @return 
     */
    double normalisescore(double score, double longueur) {
        return (100 * Math.pow(score, (1.0 / longueur)));
    }

    
    double relatedcolumnscore(int number, int cluster) {
        DirichletMixture dirichletMixture = new DirichletMixture(fileDichletMixture);

        //initialise the score(to 1 because we multiply)
        double score = 1;
        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {
                //multiply by the probability of each residue
            
                String c = sequence.Seq.get(number);
                double a = dirichletMixture.Prob((String) c, sequence.Weight, false);
                score *= a;
            }
        }

        return score;
    }

    /**
     * calculate the probability that the residues in the column are related  
     * 
     * @param number
     * @return score de la colonne
     */
    double relatedcolumnscore(int number, int cluster1, int cluster2) {
        DirichletMixture dirichletMixture = new DirichletMixture(fileDichletMixture);

        //initialise the score(to 1 because we multiply)
        double score = 1;
        //multiply by the probability of each residue

        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster1) {
                //multiply by the probability of each residue
            
                String c = sequence.Seq.get(number);
                double a = dirichletMixture.Prob((String) c, sequence.Weight, false);
                score *= a;
            }
        }
        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster2) {
                //multiply by the probability of each residue
            
                String c = sequence.Seq.get(number);
                double a = dirichletMixture.Prob((String) c, sequence.Weight, false);
                score *= a;
            }
        }
        return score;
    }
    


    /**
     * calculate sequence weights
     *

     */
    void sequenceWeights(int cluster) {

        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {
                double meanpcid = 0.0;
                int n=0;
                for (int j = 0; j < alignment.sequenceArray.size(); j++) {
                    Sequence sequence2 = (Sequence) alignment.sequenceArray.get(j);
                    if (sequence2.Cluster == cluster) {
                        if (sequence.equals(sequence2)) {}
                        else {
                        double pcid=calcpcid(sequence,sequence2);
                        meanpcid+=pcid;
                        n++;
                        }
                    }  
                }
                meanpcid /= (double)n;
                sequence.AddWeight((100.0-meanpcid)/100.0);
                //System.out.println("weight "+cluster+" "+i+" "+alignment.sequenceArray.get(i).Name+" "+((100.0-meanpcid)/100.0));
            }
        }
        // normalize weights to add to 1
        /*double tot = 0.0;
        int n=0;        
        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {
                tot+=sequence.Weight;
                n++;
 
            }

        }
        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {
                double weight=sequence.Weight/tot;
                sequence.AddWeight(weight);
            }

        }*/

    }
    
    // Apply sequence weights all equal to 1 (i.e. no sequence weighting
    /*void sequenceWeights(int cluster) {

        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {

                sequence.AddWeight(1.0);

            }
        }
 

    }*/
        
    double calcpcid(Sequence sequence1, Sequence sequence2) {
        double pcid = 0.0;
        int total=0;
        int count=0;
       
        int sizesequence1 = sequence1.Seq.size(); 
        int sizesequence2 = sequence2.Seq.size();
        for (int col = 0; col < sizesequence1 && col < sizesequence2; col++) {
            String c1 = sequence1.Seq.get(col); 
            String c2 = sequence2.Seq.get(col); 
            if (("-").equals(c1) || ("-").equals(c2)){
            }
            else {
                total++;
                if (c1.equals(c2)) count++;
            }
        }
        if(total==0) pcid=0.0;
        else pcid=100.0*(double)count/(double)total;
        return pcid;
    }
    
        /**
     * return the averaged column scores in a sliding window
     *
     * @param scoreColumns
     * @param sizeWindows return an arraylist of window scores
     */
    ArrayList<Double> scoreWindows(ArrayList scoreColumns, int sizeWindows, int numcluster) {
        ArrayList<Double> scoreWindows = new ArrayList();
        for (int col = 0; col < sizeWindows; col++) {
            scoreWindows.add(0.0);
        }
        for (int col = sizeWindows; col < scoreColumns.size() - sizeWindows; col++) {
            scoreWindows.add(scorewindow(scoreColumns, col-sizeWindows, col + sizeWindows));
        }

        return scoreWindows;
    }


    /**
     * 
     * Calculate the average score in a window
     * @param scoreColumns
     * @param start
     * @param end
     * @return average score
     */
    double scorewindow(ArrayList scoreColumns, int start, int end) {

        double sumscore = 0;

        for (int i = start; i < end + 1; i++) {
            double scorewindow = (Double) scoreColumns.get(i);
            if (scorewindow<0.0) scorewindow=0; /* test for gaps Julie */

            sumscore += scorewindow;
        }
        double meanscore = sumscore / (end - start);

        return meanscore;

    }

    /**
     * return the coreblocks
     *
     * @param scoreWindow
     * @param sizeWindows return an arraylist of coreblocks
     */
    ArrayList<int[]> coreblocks(ArrayList windowScores, ArrayList columnScores, int numcluster) {
        ArrayList<int[]> blocks = new ArrayList();
        Boolean incb = false;
        int start = 0, end = 0;
        double score = 0;
        double cscore = 0;
        
        for (int col = 0; col < windowScores.size(); col++) {
            score = (Double) windowScores.get(col);
            cscore = (Double) columnScores.get(col);
            //System.out.println("CBS"+" "+numcluster+" "+col+" "+score+" "+cscore);
            if(score > 5.0) {
                if (!incb) {
                    // start a new coreblock
                    start = end = col;  
                    incb = true; 
                } else {
                    end = col;
                }
            } else {
                if (incb) {
                    //save the coreblock
                    int[] position = {start, end}; 
                    if (end-start > 3) blocks.add(position);
                    //if (end-start > 3) System.out.println("CBADD"+" "+numcluster+" "+start+" "+end+" ");
                incb = false;
                }
            }

        }

        if (incb) {
            //save the coreblock
            int[] position = {start, end};
            if (end-start > 3) blocks.add(position);
                    //System.out.println("CBADD"+" "+numcluster+" "+start+" "+end+" ");
        }

        return blocks;
    }
    
    /**
     * return the sequence corresponding to a number
     * @param numerosequence
     * @param numerocluster
     * @return 
     */
    Sequence numbertosequence(int numerosequence, int numerocluster) {
        int numbertofind = 0;
        for (int i = 0; i < alignment.sequenceArray.size(); i++) {
            Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
            if (sequence.Cluster == cluster) {
                if (numbertofind == numerosequence) {
                    return sequence;
                }
                numbertofind++;
            }
        }

        return null;
    }

    Sequence sequencequery() {
        for (Sequence seq : alignment.sequenceArray) {
            if (seq.Name.equals(alignment.sequenceQuery)) {
                return seq;
            }
        }
        System.out.println("ERROR: Sequence query not found");
        return null;
    }

    
    boolean coreblockvscoreblock(int[] startend, int cluster1, int cluster2, double factor) {
        double related = 0;
        double unrelated = 0;
        double unrelated1 = 0;
        double unrelated2 = 0;
        double sumrelated = 0;
        double sumunrelated = 0;
        
        for (int i = startend[0]; i <= startend[1]; i++) {
            related = relatedcolumnscore(i, cluster1, cluster2);
            sumrelated+=related;
            unrelated1= relatedcolumnscore(i, cluster1);
            unrelated2= relatedcolumnscore(i, cluster2);
            unrelated = unrelated1 * unrelated2;
            sumunrelated+=unrelated;
            //System.out.println("CB"+" "+cluster1+" "+cluster2+" "+i+" "+related+" "+unrelated1+" "+unrelated2);
        }
System.out.println("CBTOT"+" "+cluster1+" "+cluster2+" "+startend[0]+" "+startend[1]+" "+sumrelated+" "+sumunrelated);

        if (sumrelated >= sumunrelated * factor) {
        //if (sumrelated*1000/(startend[1]-startend[0])>1.0) {
            return true;

        } else {
            return false;
        }


    }
    

    
    /**
     * for each coreblock, check whether it exists in the other groups
     */
    void checkcoreblock(double factor) {
        int color = 0;
        int status;
        int found1;
        int found2;

        /* first do pairwise comparisons of all overlapping blocks in two different clusters */
        ArrayList<pwcoreblock> pairwisecbs=new ArrayList<pwcoreblock>();
        for (groupcoreblock groupcb1 : alignment.coreblocks) {

                //for each coreblock, check whether it exists in the other groups
                for (int[] cba : groupcb1.coreblock) {
                    
                    for (groupcoreblock groupcb2: alignment.coreblocks) {
                        if (groupcb2.cluster != groupcb1.cluster) {

                            for (int[] cbb : groupcb2.coreblock) {

                                int[] startend=overlap(cba[0],cba[1],cbb[0],cbb[1]);             
                                if (startend[1]-startend[0]>3) {

                                    // do the core blocks match ?
                                    if (coreblockvscoreblock(startend, groupcb1.cluster, groupcb2.cluster, factor)) {
                                    //if yes, add new core block pair
                                        pwcoreblock cb = new pwcoreblock(startend[0],startend[1],groupcb1.cluster,groupcb2.cluster);
                                        pairwisecbs.add(cb);
                                    }
                                }
                            }
                        }
                    }
                }
        }

        /* then, chain the pairwise matching blocks into larger groups */
        ArrayList<chaincoreblock> chaincbs=new ArrayList<chaincoreblock>();
        for (pwcoreblock pwcb : pairwisecbs) {
            status=0;
            /* check whether core block pair overlaps with an existing chain */
            for (chaincoreblock chain : chaincbs) {
                int[] startend=overlap(pwcb.debut,pwcb.fin,chain.debut,chain.fin); 
                if(startend[1]-startend[0]>3) { 
                    // if both clusters are already in the chain, no need to do anything
                    found1=found2=0;
                    for (int cl : chain.cluster) {
                        if(pwcb.cluster1 == cl) found1=1;
                        if(pwcb.cluster2 == cl) found2=1;
                    }
                    if (found1==1 && found2==1) {
                        status=1;
                        break;
                    }
                    // add missing cluster to chain 
                    for (int cl : chain.cluster) {
                        if(pwcb.cluster1 == cl || pwcb.cluster2 == cl) {
                            // create new chains for before and after the new block if necessary
                            if(startend[0]-1-chain.debut> 3) {
                                chaincoreblock cb;
                                cb = new chaincoreblock(chain.debut,startend[0]-1);
                                for (int cl1 : chain.cluster) cb.AddCluster(cl1);
                                chaincbs.add(cb);
                            }
                            if(chain.fin-1-startend[1]> 3) {
                                chaincoreblock cb;
                                cb = new chaincoreblock(startend[1],chain.fin-1);
                                for (int cl1 : chain.cluster) cb.AddCluster(cl1);
                                chaincbs.add(cb);
                            }
                            // add new block to chain
                            chain.debut=startend[0];
                            chain.fin=startend[1];                           
                            if(pwcb.cluster1 == cl) {        
                                chain.AddCluster(pwcb.cluster2);
                                status=1;
                                break;
                            }    
                            else if(pwcb.cluster2 == cl) {
                                chain.AddCluster(pwcb.cluster1);
                                status=2;
                                break;
                            }
                        }
                    }
                }
                if (status!=0) break;
            }
            
            if (status==0) {
                chaincoreblock cb;
                cb = new chaincoreblock(pwcb.debut,pwcb.fin);
                cb.AddCluster(pwcb.cluster1);
                cb.AddCluster(pwcb.cluster2);
                chaincbs.add(cb);
            }
        }
        
        // add core blocks for sequences in chained clusters
        color=0;
        for ( chaincoreblock chain : chaincbs ) {
            for (int i=0; i<chain.cluster.size(); i++) {
                int cl = chain.cluster.get(i);
                int [] startend = {chain.debut,chain.fin};
                if(chain.fin-chain.debut>3) ScoresegmentsAgainstcluster(startend, cl, cl, color);
            }
            color++;
        }
        
        // finally, add the single core blocks found in only one cluster that do not overlap with any chains
        
        //for each cluster
        for (groupcoreblock groupcb : alignment.coreblocks) {

            int sizesequence = (((alignment.sequenceArray).get(1)).Seq).size();
            int[] blocks; 
            blocks= new int[sizesequence];
            for (int c = 0; c < sizesequence; c++) {
               blocks[c]=0;
            }
            
            // mark the core blocks for the cluster
            for (int[] cba : groupcb.coreblock) {
                for (int c=cba[0]; c<cba[1];c++) blocks[c]=1;
            }
            // mark the chained core blocks
            for ( chaincoreblock chain : chaincbs ) {
                for (int i=0; i<chain.cluster.size(); i++) {
                    int cl = chain.cluster.get(i);
                    if (groupcb.cluster==cl) {
                        for (int c=chain.debut; c<chain.fin;c++) blocks[c]=2;
                    }
                }
            }
            
            Boolean incb=false;
            int start = 0;
            int end = 0;
            for (int c = 0; c < sizesequence; c++) {
                int score = blocks[c];
                if(score == 1) {
                    if (!incb) {
                        // start a new coreblock
                        start = end = c;  
                        incb = true; 
                    } else {
                        end = c;
                    }
                } else {
                    if (incb) {
                        //save the coreblock
                        int[] startend = {start, end}; 
                        if (end-start > 3) {
                            ScoresegmentsAgainstcluster(startend, groupcb.cluster, groupcb.cluster, color);
                            color++;
                        }
                    incb = false;
                    }
                }

            }

            if (incb) {
                //save the coreblock
                int[] startend = {start, end};
                if (end-start > 3) {
                    ScoresegmentsAgainstcluster(startend, groupcb.cluster, groupcb.cluster, color);
                    color++;
                }
            }
        }
    }

    /**
     * calculate the overlap between two segments 

     */
    int[] overlap(int s1, int e1, int s2, int e2){

        int[] startend = {0,0};

        if((s1 >= s2 && s1 <= e2) || (e1 >= s2 && e1 <= e2) || (s2 >= s1 && s2 <= e1) || (e2 >= s1 && e2 <= e1) ) {
            int start;
            int end;
            if (s1 > s2) {
                start = s1;
            } else {
                start = s2;
            }
            if (e1 > e2) {
                end = e2;
            } else {
                end = e1;
            }
            startend[0]=start;
            startend[1]=end;
        }
        
        return startend;
    }
        
    /**
     * compare the sequences in a coreblock  to a coreblock in the query 
     * @param startend
     * @param querycluster
     * @param clustertocompare 
     */
    void ScoresegmentsAgainstcluster(int[] startend, int querycluster, int clustertocompare, int color) {
        Boolean status;
        for (Sequence seq : alignment.sequenceArray) {
            if (seq.Cluster == clustertocompare) {
                status=ScoresegmentAgainstcluster(startend, querycluster, seq, color);
            }
        }
    }

    /**
     * compare the coreblock in a sequence to a coreblock in the query
     * @param startend 
     * @param querycluster
     * @param seqtocompare //sequence que l'on compare
     * @return 
     */
    Boolean ScoresegmentAgainstcluster(int[] startend, int querycluster, Sequence seqtocompare, int color) {
        double scoreseg = 1;
        double score;
        //score of a random segment 
        double random=randomsegment(seqtocompare, startend[0], startend[1]);
        for (int c = startend[0]; c < startend[1] + 1; c++) {
            //
            DirichletMixture dirichletMixture = new DirichletMixture(fileDichletMixture);

            // if the sequence belongs to the query cluster, remove it from the column  

            for (int i = 0; i < alignment.sequenceArray.size(); i++) {
                Sequence sequence = (Sequence) alignment.sequenceArray.get(i);
                if (sequence.Cluster == querycluster) {
                    
                    if(sequence.equals(seqtocompare)) {}
                    else {
                        String ch = sequence.Seq.get(c);
                        dirichletMixture.Prob((String) ch, sequence.Weight, false);
                    }
                }
            }

            score = dirichletMixture.Prob(seqtocompare.Seq.get(c), seqtocompare.Weight, false);
            scoreseg *= score;
        }
        
//System.out.println("SIBIS "+startend[0]+" "+ startend[1]+" "+seqtocompare.Name+" "+scoreseg+" "+random+" "+scoreseg/random+" "+color);
        if (scoreseg > random) {
            seqtocompare.AddCoreBlock(startend[0], startend[1], normalisescore(scoreseg, (startend[1] - startend[0])), color, seqtocompare.Cluster);
            return true;
        }
        return false;
    }
    

    /**
     * 
     * @param seq
     * @param debut
     * @param fin
     * @return 
     */
    
    double randomsegment(Sequence seq, int debut, int fin){
        /*int length;
        double randomscore;
       
        length=fin-debut;
        randomscore=length*log(20);
        return randomscore;*/
        double randomscore=1;
        for (int i=debut;i<=fin;i++){
            randomscore*=randomaa(seq.Seq.get(i));
        }
        return randomscore;
    }
 
    
    /**
     * renvois le score d'un résidu
     * @param lettre
     * @return score d'un résidu 
     */
    double randomaa(String aa) {
        double scoreaa =0.05;
       
        String[] ArrayOfResidues = {"A", "C", "D", "E", "F", "G", "H", "I", "K", "L", "M", "N", "P", "Q", "R", "S", "T", "V", "W", "Y"};
        double[] Arrayscore = {0.0825,0.0137,0.0545 ,0.0675 ,0.0386 ,0.0707,0.0227,0.0596,0.0584,0.0966,0.0242,0.0406,0.0470,0.0393,0.0553 ,0.0656,0.0534,0.0687 ,0.0108,0.0292};
        for (int i=0;i<ArrayOfResidues.length;i++){
            if (aa.equals(ArrayOfResidues[i])){
                scoreaa=Arrayscore[i];
            }
        }
        
               return scoreaa;     
        }
        
        
    
}

