#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "leon.h"

#define MAXCOLORTEXTLEN 30
#define MAXCOLORS 24
#define MAXSITETYPE 100
#define MAXJAL 1000

int color_lut[MAXCOLORS][3]={
0,0,225,
192,192,0,
128,0,0,
0,225,225,
0,128,128,
64,128,0,
192,128,0,
192,128,192,
0,150,192,
64,64,225,
192,225,0,
255,192,0,
225,0,128,
0,225,192,
150,0,225,
0,192,225,
164,225,64,
225,192,64,
225,64,192,
64,64,225,
192,64,225,
192,225,64,
128,0,128,
192,0,192,
};

typedef struct {
	char filename[FILENAMELEN+1];
	char name[MAXTITLES+1];
	char framename[MAXTITLES+1];
	FILE *fd;
	int width;
	int height;
} SVG;

typedef struct {
	char text[MAXTITLES+1];
	int nentries;
	int index;
} FEATURE;

static FILE *  open_outfile(char *root, char *file_extension, char *out_name);
static void write_tophtml(FILE *fd, char *outfileroot);
static void write_jaltext(FILE *fd,ALN mult_aln,int nfeatures,FEATURE *features);
static int count_network_nodes(ALNPTR mult_aln,int ftype);
static void get_network_nodes(ALN mult_aln,int ftype,GROUP_NODE *nodes,int nnodes);
static void phylo_index(ALNPTR mult_aln);
static void assign_colors_to_ft(ALNPTR mult_aln,int ft);
static void get_color(int r,int g,int b,char *color);
static void warn_color(int r,int g,int b,char *color);
static void prop_color(int r,int g,int b,char *color);

	int nnetwork_nodes;
	GROUP_NODE *network_nodes;

int main(int argc, char **argv)
{
	FILE *ifd,*fd;
	char infile[FILENAMELEN+1];
	char outfile[FILENAMELEN+1];
	char outfileroot[FILENAMELEN+1];
	char outfiletfa[FILENAMELEN+1];
	char outfiletxt[FILENAMELEN+1];
	FEATURE features[MAXFTTYPE];
	char format;
	int  n,i,j,k,l,nentries,nfeatures,ires,iseq=0;
	int max_flen;
	Boolean found;
	int maxnames;
	ALN mult_aln;
	ALN cons_aln;
	OPT opt;
	int aln_header_height;
	float aln_font_height;
	float aln_font_width;
	float button_font_height;
	float button_font_width;
	int xoffset;

	if(argc!=3) {
		fprintf(stderr,"Usage: %s probe_file out_file\n",argv[0]);
		exit(1);
	}
        strcpy(infile,argv[1]);
        strcpy(outfile,argv[2]);

/* get the output filename without the path information */
	for(i=strlen(outfile);i>=0;i--)
		if(outfile[i]=='/') break;
	if(i!=0) i++;
	for(j=i;j<strlen(outfile);j++)
		outfileroot[j-i]=outfile[j];
	outfileroot[j-i]='\0';

fprintf(stdout,"%s %s\n",outfile,outfileroot);
        init_options(&opt);
        (*opt.alnout_opt).output_clustal=FALSE;
        (*opt.alnout_opt).output_tfa=TRUE;


	/* read in the sequences */
        seq_input(infile,opt.explicit_type,FALSE,&mult_aln);
        if(mult_aln.nseqs<=0) {
                error("No sequences in %s\n",infile);
                exit(1);
        }

	maxnames=0;
	for(i=0;i<mult_aln.nseqs;i++) {
		n=strlen(mult_aln.seqs[i].name);
		if(n>maxnames) maxnames=n;
	}

/* count the number of feature buttons needed */
        nfeatures=0;
        max_flen=0;
        for(i=0;i<MAXFTTYPE;i++) {
/*if(i==VARSPLIC || i==SWDOMAIN || i==SEQERRBLOCK) continue;*/
/*if(i==VARSPLIC || i==SEQERRBLOCK) continue;*/
if(i==VARSPLIC) continue;
                nentries=0;
                found=FALSE;
                for(j=0;j<mult_aln.nseqs;j++)
                        if(mult_aln.ft[j].nentries[i]>0) {
                                nentries+=mult_aln.ft[j].nentries[i];
                                if(found==FALSE) {
                                        if(i==PHYLOBLOCK)
                                        strcpy(features[nfeatures].text,"PHYLOGENY");
                                        else
                                        strcpy(features[nfeatures].text,mult_aln.ft[j].data[i][0].type);
                                        features[nfeatures].index=i;
                                        l=strlen(features[nfeatures].text);
                                        if(l>max_flen) max_flen=l;
                                        found=TRUE;
                                }
                        }
                features[nfeatures].nentries=nentries;
                if(nentries>0)
                        nfeatures++;
        }

/* get the phylogenetic distribution */
	phylo_index(&mult_aln);

	nnetwork_nodes=count_network_nodes(&mult_aln,PHYLOBLOCK);
	network_nodes=(GROUP_NODE *)ckalloc((nnetwork_nodes+1)*sizeof(GROUP_NODE));
	get_network_nodes(mult_aln,PHYLOBLOCK,network_nodes,nnetwork_nodes);

	for(j=0;j<mult_aln.nseqs;j++) {
		for(k=0;k<mult_aln.ft[j].nentries[PHYLOBLOCK];k++) {
			mult_aln.ft[j].data[PHYLOBLOCK][k].color=network_nodes[mult_aln.ft[j].data[PHYLOBLOCK][k].color].color;
/*
			strcat(mult_aln.ft[j].data[PHYLOBLOCK][k].name," ");
			strcat(mult_aln.ft[j].data[PHYLOBLOCK][k].name,network_nodes[mult_aln.ft[j].data[PHYLOBLOCK][k].color].taxon);
*/
			for(i=0;i<strlen(mult_aln.ft[j].data[PHYLOBLOCK][k].name);i++)
				if(mult_aln.ft[j].data[PHYLOBLOCK][k].name[i]==' ')
					mult_aln.ft[j].data[PHYLOBLOCK][k].name[i]='_';
		}
	}

/* assign colors */
        assign_colors_to_ft(&mult_aln,SITE);
        assign_colors_to_ft(&mult_aln,MODRES);
        assign_colors_to_ft(&mult_aln,SIGNAL);

/* write the jalview text */
	fd=open_outfile(outfile,".txt",outfiletxt);
	fprintf(stdout,"creating %s\n",outfiletxt);
	write_jaltext(fd,mult_aln,nfeatures,features);
	fclose(fd);

/* write out the sequences */
	for(i=0;i<mult_aln.nseqs;i++)
		mult_aln.seqs[i].output_index=i;

        strcpy((*opt.alnout_opt).tfa_outname, outfile);
        strcat((*opt.alnout_opt).tfa_outname, ".tfa");

        if(!open_alignment_output(infile,opt.alnout_opt)) exit(1);
        create_alignment_output(mult_aln,*opt.alnout_opt);

	exit(0);
}

static void assign_colors_to_ft(ALNPTR mult_aln,int ft)
{
        int i,j,k;
        char type_list[MAXSITETYPE][MAXFTNAMES+1];
        Boolean found;
        int ntypes;
        char name[MAXFTNAMES+1];
        char name1[MAXFTNAMES+1];

        ntypes=0;
        for(i=0;i<mult_aln->nseqs;i++) {
                for(j=0;j<mult_aln->ft[i].nentries[ft];j++) {
			if(strncmp(mult_aln->ft[i].data[ft][j].name,"PROP_",5)==0)
				strcpy(name1,&mult_aln->ft[i].data[ft][j].name[5]);
			else if(strncmp(mult_aln->ft[i].data[ft][j].name,"PRED_",5)==0)
				strcpy(name1,&mult_aln->ft[i].data[ft][j].name[5]);
			else if(strncmp(mult_aln->ft[i].data[ft][j].name,"WARN_",5)==0)
				strcpy(name1,&mult_aln->ft[i].data[ft][j].name[5]);
			else if(strncmp(mult_aln->ft[i].data[ft][j].name,"ERROR_",6)==0)
				strcpy(name1,&mult_aln->ft[i].data[ft][j].name[6]);
			else
				strcpy(name1,mult_aln->ft[i].data[ft][j].name);
                        for(k=0;k<strlen(name1);k++) {
                                if(isspace(name1[k])) break;
                                name[k]=name1[k];
                        }
                        name[k]='\0';
                        found=FALSE;
                        for(k=0;k<ntypes;k++)
                                if(strcmp(name,type_list[k])==0) {
                                        mult_aln->ft[i].data[ft][j].color=k;
                                        found=TRUE;
                                        break;
                                }
                        if(found==FALSE) {
                                strcpy(type_list[ntypes],name);
                                mult_aln->ft[i].data[ft][j].color=ntypes;
                                ntypes++;
                        }
                        if(ntypes>=MAXSITETYPE) {
                                error("too many sites");
                                break;
                        }
                        if(ntypes>=MAXSITETYPE) break;
                }
                if(ntypes>=MAXSITETYPE) break;
        }
}

static void write_tophtml(FILE *fd, char *outfileroot)
{
	fprintf(fd,"<html>\n");

	fprintf(fd,"<body>\n");

	fprintf(fd,"<center>\n");
  	fprintf(fd,"<p> \n");
    	fprintf(fd,"<applet code =\"jalview.bin.JalviewLite\" height=\"100\" width =\"200\" name=\"jalview\" archive=\"http://lbgi.igbmc.fr/Jalview/jalviewApplet.jar\">\n");
      	fprintf(fd,"<param name=\"file\" value=\"%s.tfa\">\n",outfileroot);
      	fprintf(fd,"<param name=\"features\" value=\"%s.txt\">\n",outfileroot);
      	fprintf(fd,"<param name=\"embedded\" value=\"false\">\n");
      	fprintf(fd,"<param name=\"windowHeight\" value=\"600\">\n");
      	fprintf(fd,"<param name=\"windowWidth\" value=\"900\">\n");

    	fprintf(fd,"</applet>\n");
  	fprintf(fd,"</p>\n");

	fprintf(fd,"</center>\n");

	fprintf(fd,"</body>\n");
	fprintf(fd,"</html>\n");
}

static void write_jaltext(FILE *fd,ALN mult_aln,int nfeatures,FEATURE *features)
{
	int ii,i,j,k,n;
	int cstart,cend;
	int width;
	int icolor;
	char name[MAXFTNAMES+1];
	char name1[MAXFTNAMES+1];
	char name2[MAXFTNAMES+1];
	char stmp[MAXFTNAMES+1];
	char color[MAXCOLORTEXTLEN];
	char sname[MAXJAL][MAXFTNAMES+1];
	int nsname;
	int r,g,b;
	Boolean found;
	Boolean prop;

	nsname=0;
	for(ii=0;ii<nfeatures;ii++) {
		i=features[ii].index;
		for(j=0;j<mult_aln.nseqs;j++) {
			for(k=0;k<mult_aln.ft[j].nentries[i];k++) {
				if(strcmp(mult_aln.ft[j].data[i][k].name,"LBLOCK")==0) continue;
				prop=FALSE;
				if(strncmp(mult_aln.ft[j].data[i][k].name,"PROP_",5)==0) {
					strcpy(name,&mult_aln.ft[j].data[i][k].name[5]);
					prop=TRUE;
				}
				else if(strncmp(mult_aln.ft[j].data[i][k].name,"PRED_",5)==0)
					strcpy(name,&mult_aln.ft[j].data[i][k].name[5]);
				else if(strncmp(mult_aln.ft[j].data[i][k].name,"WARN_",5)==0)
					strcpy(name,&mult_aln.ft[j].data[i][k].name[5]);
				else if(strncmp(mult_aln.ft[j].data[i][k].name,"ERROR_",6)==0)
					strcpy(name,&mult_aln.ft[j].data[i][k].name[6]);
				else
					strcpy(name,mult_aln.ft[j].data[i][k].name);
				if((strcmp(name,"REGION")==0) || (strcmp(name,"SBLOCK")==0) || (strcmp(name,"COIL")==0) || (strcmp(name,"SEQERR")==0)) {
					sprintf(stmp,"%d",mult_aln.ft[j].data[i][k].color+1);
					strcat(name,stmp);
				}
				else if (i==VARIANT || i==REPEAT || i==SIGNAL || i==TRANSMEM) {
					strcpy(name,features[ii].text);
				}
                        	for(n=0;n<strlen(name);n++) {
                               		if(isspace(name[n])) break;
                               		name1[n]=name[n];
                        	}
                        	name1[n]='\0';
				found=FALSE;
				for(n=0;n<nsname;n++) {
					if(strcmp(sname[n],name1)==0) {
						found=TRUE;
						break;
					}
				}
				if(found==FALSE) {
					if(mult_aln.ft[j].data[i][k].color>=MAXCOLORS) {
						/*strcpy(color,"gray");*/
						/*r=g=b=128;*/
						icolor=mult_aln.ft[j].data[i][k].color-(mult_aln.ft[j].data[i][k].color/MAXCOLORS)*MAXCOLORS;
						r=color_lut[icolor][0];
						g=color_lut[icolor][1];
						b=color_lut[icolor][2];
					}
					else if (i==VARIANT) {
						r=225;
						g=0;
						b=225;
					}
					else if (i==STRUCT) {
						if(strncmp(name1,"HELIX",5)==0) {
							r=192;
							g=0;
							b=0;
						}
						else {
							r=0;
							g=128;
							b=0;
						}
					}
					else {
						r=color_lut[mult_aln.ft[j].data[i][k].color][0];
						g=color_lut[mult_aln.ft[j].data[i][k].color][1];
						b=color_lut[mult_aln.ft[j].data[i][k].color][2];
					}
					strcpy(sname[nsname],name1);
					nsname++;
					get_color(r,g,b,color);
					fprintf(fd,"%s\t%s\n",name1,color);
					prop_color(r,g,b,color);
					fprintf(fd,"PRED_%s\t%s\n",name1,color);
					prop_color(r,g,b,color);
					fprintf(fd,"PROP_%s\t%s\n",name1,color);
					warn_color(r,g,b,color);
					fprintf(fd,"WARN_%s\t%s\n",name1,color);
					warn_color(r,g,b,color);
					fprintf(fd,"ERROR_%s\t%s\n",name1,color);
				}
			}
		}
	}
	fprintf(fd,"\n");

	for(ii=0;ii<nfeatures;ii++) {
		i=features[ii].index;
		i=features[ii].index;
		fprintf(fd,"STARTGROUP\t%s\n",features[ii].text);
		for(j=0;j<mult_aln.nseqs;j++) {
			for(k=0;k<mult_aln.ft[j].nentries[i];k++) {
				if(strcmp(mult_aln.ft[j].data[i][k].name,"LBLOCK")==0) continue; 
				if(strncmp(mult_aln.ft[j].data[i][k].name,"PROP_",5)==0) 
					strcpy(name,&mult_aln.ft[j].data[i][k].name[5]);
				else if(strncmp(mult_aln.ft[j].data[i][k].name,"PRED_",5)==0)
					strcpy(name,&mult_aln.ft[j].data[i][k].name[5]);
				else if(strncmp(mult_aln.ft[j].data[i][k].name,"WARN_",5)==0)
					strcpy(name,&mult_aln.ft[j].data[i][k].name[5]);
				else if(strncmp(mult_aln.ft[j].data[i][k].name,"ERROR_",6)==0)
					strcpy(name,&mult_aln.ft[j].data[i][k].name[6]);
				else
					strcpy(name,mult_aln.ft[j].data[i][k].name);
                       		for(n=0;n<strlen(mult_aln.ft[j].data[i][k].name);n++) {
                               		if(isspace(mult_aln.ft[j].data[i][k].name[n])) break;
                               		name2[n]=mult_aln.ft[j].data[i][k].name[n];
                       		}
                       		name2[n]='\0';
                       		for(n=0;n<strlen(name);n++) {
                               		if(isspace(name[n])) break;
                               		name1[n]=name[n];
                       		}
                       		name1[n]='\0';
				if((strcmp(name,"REGION")==0) || (strcmp(name,"SBLOCK")==0) || (strcmp(name,"COIL")==0) || (strcmp(name,"SEQERR")==0)) {
					sprintf(stmp,"%d",mult_aln.ft[j].data[i][k].color+1);
					strcat(name1,stmp);
					strcat(name2,stmp);
				}
				else if (i==VARIANT || i==REPEAT || i==SIGNAL || i==TRANSMEM) {
					strcpy(name1,features[ii].text);
				}
/* check that a color has been declared for this feature */
				found=FALSE;
				for(n=0;n<nsname;n++) {
					if(strcmp(name1,sname[n])==0) {
						found=TRUE;
						break;
					}
				}
				if (i==VARIANT || i==REPEAT || i==SIGNAL || i==TRANSMEM) {
					strcpy(name2,features[ii].text);
				}
				strcpy(name1,mult_aln.ft[j].data[i][k].name);
/*
				strcpy(name,mult_aln.ft[j].data[i][k].name);
                       		for(n=0;n<strlen(name);n++) {
                               		if(isspace(name[n])) break;
                               		name1[n]=name[n];
                       		}
                       		name1[n]='\0';
*/
				if(found==FALSE) {
					fprintf(stdout,"Error: %s %d %d %s has no color defined\n",mult_aln.seqs[j].name,mult_aln.ft[j].data[i][k].start+1,mult_aln.ft[j].data[i][k].end+1,name2);
				}
				else {
				/*fprintf(fd,"%s\t%s\t-1\t%d\t%d\t%s\n",mult_aln.ft[j].data[i][k].name,mult_aln.seqs[j].name,mult_aln.ft[j].data[i][k].start+1,mult_aln.ft[j].data[i][k].end+1,name2);*/
				fprintf(fd,"%s\t%s\t-1\t%d\t%d\t%s\n",name1,mult_aln.seqs[j].name,mult_aln.ft[j].data[i][k].start+1,mult_aln.ft[j].data[i][k].end+1,name2);
				}
			}
		}
		fprintf(fd,"ENDGROUP\t%s\n\n",features[ii].text);
	}
}

static void get_color(int r,int g,int b,char *color)
{
	int i;

	sprintf(color,"%2X%2X%2X",r,g,b);
	for(i=0;i<strlen(color);i++)
		if(isspace(color[i])) color[i]='0';
}

static void prop_color(int r,int g,int b,char *color)
{
	int i;

	r=r+64;
	g=g+64;
	b=b+64;
	if(r>225) r=225;
	if(g>225) g=225;
	if(b>225) b=225;
	sprintf(color,"%2X%2X%2X",r,g,b);
	for(i=0;i<strlen(color);i++)
		if(isspace(color[i])) color[i]='0';
}

static void warn_color(int r,int g,int b,char *color)
{
	int i;

	r=r-64;
	g=g-64;
	b=b-64;
	if(r<0) r=0;
	if(g<0) g=0;
	if(b<0) b=0;
	sprintf(color,"%2X%2X%2X",r,g,b);
	for(i=0;i<strlen(color);i++)
		if(isspace(color[i])) color[i]='0';
}

static void phylo_index(ALNPTR mult_aln)
{
	int i;
	int arc,bac,euk,vir,unk;
	int narc,nbac,neuk,nvir;

	narc=nbac=neuk=nvir=0;
        for(i=0;i<mult_aln->nseqs;i++) {
                switch(mult_aln->seqs[i].lifedomain[0]) {
                        case 'A': case 'a':
                                narc++;
                        break;
                        case 'B': case 'b':
                                nbac++;
                        break;
                        case 'E': case 'e':
                                neuk++;
                        break;
                        case 'V': case 'v':
                                nvir++;
                        break;
                }

        }

	arc=bac=euk=vir=unk=0;
        for(i=0;i<mult_aln->nseqs;i++) {
                switch(mult_aln->seqs[i].lifedomain[0]) {
                        case 'A': case 'a':
				mult_aln->seqs[i].output_index=arc;
                                arc++;
                        break;
                        case 'B': case 'b':
				mult_aln->seqs[i].output_index=narc+bac;
                                bac++;
                        break;
                        case 'E': case 'e':
				mult_aln->seqs[i].output_index=narc+nbac+euk;
                                euk++;
                        break;
                        case 'V': case 'v':
				mult_aln->seqs[i].output_index=narc+nbac+neuk+vir;
                                vir++;
                        break;
			default:
				mult_aln->seqs[i].output_index=narc+nbac+neuk+nvir+unk;
				unk++;
                }

        }
}

static int count_network_nodes(ALNPTR mult_aln,int ftype)
{
	int i,j,k;
	int maxnode,nnodes;
	int *nodes;
	Boolean found;

	maxnode=0;
	for(i=0;i<mult_aln->nseqs;i++) {
		for(j=0;j<mult_aln->ft[i].nentries[ftype];j++) {
			if(mult_aln->ft[i].data[ftype][j].color>maxnode) {
				maxnode=mult_aln->ft[i].data[ftype][j].color;
			}
		}
	}

	nnodes=maxnode+1;

	return nnodes;
}

/* any feature can be used for the nodes, but the color code has to be unique for each region. */
/* i.e. one sequence should not have two regions with the same color */

static void get_network_nodes(ALN mult_aln,int ftype,GROUP_NODE *nodes,int nnodes)
{
	int i,j,k,n;
	Boolean found;
	char **node_dictionary;

/* cound the number of sequences in each node */
	for(i=0;i<mult_aln.nseqs;i++) {
		for(j=0;j<mult_aln.ft[i].nentries[ftype];j++) {
			k=mult_aln.ft[i].data[ftype][j].color;
			nodes[k].nseqs++;
		}
	}

/* now allocate memory and get sequence info */
	for(i=0;i<nnodes;i++) {
		nodes[i].seqs=(int *)ckalloc((nodes[i].nseqs+1)*sizeof(int));
		nodes[i].nseqs=0;
	}

	for(i=0;i<mult_aln.nseqs;i++) {
		for(j=0;j<mult_aln.ft[i].nentries[ftype];j++) {
			k=mult_aln.ft[i].data[ftype][j].color;
			nodes[k].start=mult_aln.ft[i].data[ftype][j].start;
			nodes[k].end=mult_aln.ft[i].data[ftype][j].end;
			nodes[k].seqs[nodes[k].nseqs]=i;
			nodes[k].nseqs++;
		}
	}

/* get the consensus phylogeny for each node */
	for(k=0;k<nnodes;k++) {
		if(nodes[k].nseqs<=0) continue;

/* get the common ancestor taxon for the node */
		common_denominator(mult_aln,&nodes[k]);

	}

/* color the node according to the phylogeny */
	node_dictionary=(char **)ckalloc((nnodes+11)*sizeof(char *));
	for(k=0;k<nnodes+11;k++) 
		node_dictionary[k]=(char *)ckalloc((MAXORGANISMS+1)*sizeof(char));

	strcpy(node_dictionary[0],"Eukaryota");
	strcpy(node_dictionary[1],"Archaea");
	strcpy(node_dictionary[2],"Bacteria");
	strcpy(node_dictionary[3],"Virus");
	strcpy(node_dictionary[4],"AB");
	strcpy(node_dictionary[5],"AE");
	strcpy(node_dictionary[6],"AV");
	strcpy(node_dictionary[7],"BE");
	strcpy(node_dictionary[8],"BV");
	strcpy(node_dictionary[9],"EV");
	n=10;
	for(k=0;k<nnodes;k++) {
		if(nodes[k].nseqs<=0) continue;
		found=FALSE;
		for(i=0;i<n;i++) {
			if(strcmp(nodes[k].taxon,node_dictionary[i])==0) {
				nodes[k].color=i;
				found=TRUE;
				break;
			}
		}
		if(found==FALSE) {
			strcpy(node_dictionary[n],nodes[k].taxon);
			nodes[k].color=n;
			n++;
		}
		fprintf(stdout,"%d: %s %d %d %d %d (%d %d %d %d) %s\n",k+1,mult_aln.seqs[nodes[k].seqs[0]].name,nodes[k].color,nodes[k].nseqs,nodes[k].start,nodes[k].end,nodes[k].arc,nodes[k].bac,nodes[k].euk,nodes[k].vir,nodes[k].taxon);
	}
}


static FILE *  open_outfile(char *root, char *file_extension, char *out_name)
{
        FILE * file_handle;

        strcpy(out_name,root);
        strcat(out_name,file_extension);

        file_handle = open_explicit_file(out_name);
        return file_handle;
}


